@@ -2,14 +2,13 @@ import logging
 import os
 import re
 import shutil
-import sys
-from abc import ABC, abstractmethod
-from pprint import pformat, pprint
+# import sys
+from pprint import pformat
+# from pprint import pprint
 from tempfile import gettempdir
 from typing import Any, Callable, Optional, Tuple, Union
 
 import click
-import requests
 from mutagen.flac import FLAC, Picture
 from mutagen.id3 import APIC, ID3, ID3NoHeaderError
 from pathvalidate import sanitize_filename, sanitize_filepath
@@ -34,7 +33,7 @@ from .metadata import TrackMetadata
 from .utils import (
     clean_format,
     decrypt_mqa_file,
-    quality_id,
+    get_quality_id,
     safe_get,
     tidal_cover_url,
     tqdm_download,
@@ -43,10 +42,10 @@ from .utils import (
 logger = logging.getLogger(__name__)
 
 TIDAL_Q_MAP = {
-    "LOW": 4,
-    "HIGH": 5,
-    "LOSSLESS": 6,
-    "HI_RES": 7,
+    "LOW": 0,
+    "HIGH": 1,
+    "LOSSLESS": 2,
+    "HI_RES": 3,
 }
 
 # used to homogenize cover size keys
@@ -228,7 +227,7 @@ class Track:
         else:
             raise InvalidSourceError(self.client.source)
 
-        if dl_info.get("enc_key"):
+        if isinstance(dl_info, dict) and dl_info.get("enc_key"):
             decrypt_mqa_file(temp_file, self.final_path, dl_info["enc_key"])
         else:
             shutil.move(temp_file, self.final_path)
@@ -251,7 +250,7 @@ class Track:
 
         assert hasattr(self, "cover_url"), "must pass cover_url parameter"
 
-        self.cover_path = os.path.join(self.folder, f"cover{hash(self.cover_url)}.jpg")
+        self.cover_path = os.path.join(self.folder, f"cover{hash(self.id)}.jpg")
         logger.debug(f"Downloading cover from {self.cover_url}")
         if not os.path.exists(self.cover_path):
             tqdm_download(self.cover_url, self.cover_path)
@@ -293,9 +292,7 @@ class Track:
         :raises IndexError
         """
 
-        logger.debug(pos)
         tracklist = cls._get_tracklist(album, client.source)
-        logger.debug(len(tracklist))
         track = tracklist[pos]
         meta = TrackMetadata(album=album, track=track, source=client.source)
         return cls(client=client, meta=meta, id=track["id"])
@@ -356,18 +353,18 @@ class Track:
         if album_meta is not None:
             self.meta.add_album_meta(album_meta)  # extend meta with album info
 
-        if self.quality in (6, 7, 27):
+        if self.quality in (2, 3, 4):
             self.container = "FLAC"
             logger.debug("Tagging file with %s container", self.container)
             audio = FLAC(self.final_path)
-        elif self.quality == 5:
+        elif self.quality == 1:
             self.container = "MP3"
             logger.debug("Tagging file with %s container", self.container)
             try:
                 audio = ID3(self.final_path)
             except ID3NoHeaderError:
                 audio = ID3()
-        elif self.quality == 4:  # tidal and deezer
+        elif self.quality == 0:  # tidal and deezer
             # TODO: add compatibility with MP4 container
             raise NotImplementedError("Qualities < 320kbps not implemented")
         else:
@@ -490,7 +487,7 @@ class Track:
         return f"{self['artist']} - {self['title']}"
 
 
-class Tracklist(list, ABC):
+class Tracklist(list):
     """A base class for tracklist-like objects.
 
     Implements methods to give it dict-like behavior. If a Tracklist
@@ -509,20 +506,6 @@ class Tracklist(list, ABC):
     IndexError
     """
 
-    def __getitem__(self, key: Union[str, int]):
-        if isinstance(key, str):
-            return getattr(self, key)
-
-        if isinstance(key, int):
-            return super().__getitem__(key)
-
-    def __setitem__(self, key: Union[str, int], val: Any):
-        if isinstance(key, str):
-            setattr(self, key, val)
-
-        if isinstance(key, int):
-            super().__setitem__(key, val)
-
     def get(self, key: Union[str, int], default: Optional[Any]):
         if isinstance(key, str):
             if hasattr(self, key):
@@ -579,7 +562,7 @@ class Tracklist(list, ABC):
         :type quality: int
         :rtype: Union[Picture, APIC]
         """
-        cover_type = {5: APIC, 6: Picture, 7: Picture, 27: Picture}
+        cover_type = {1: APIC, 2: Picture, 3: Picture, 4: Picture}
 
         cover = cover_type.get(quality)
         if cover is Picture:
@@ -599,12 +582,16 @@ class Tracklist(list, ABC):
 
         return cover_obj
 
+    def download_message(self):
+        click.secho(
+            f"\nDownloading {self.title} ({self.__class__.__name__})\n",
+            fg="blue",
+        )
+
     @staticmethod
-    @abstractmethod
     def _parse_get_resp(item, client):
         pass
 
-    @abstractmethod
     def download(self, **kwargs):
         pass
 
@@ -621,9 +608,23 @@ class Tracklist(list, ABC):
 
         return album
 
+    def __getitem__(self, key: Union[str, int]):
+        if isinstance(key, str):
+            return getattr(self, key)
+
+        if isinstance(key, int):
+            return super().__getitem__(key)
+
+    def __setitem__(self, key: Union[str, int], val: Any):
+        if isinstance(key, str):
+            setattr(self, key, val)
+
+        if isinstance(key, int):
+            super().__setitem__(key, val)
+
 
 class Album(Tracklist):
-    """Represents a downloadable Qobuz album.
+    """Represents a downloadable album.
 
     Usage:
 
@@ -665,7 +666,7 @@ class Album(Tracklist):
         for k, v in self._parse_get_resp(self.meta, self.client).items():
             setattr(self, k, v)  # prefer to __dict__.update for properties
 
-        if not self.get("streamable", False):  # Typing's sake
+        if not self.get("streamable", False):
             raise NonStreamable(f"This album is not streamable ({self.id} ID)")
 
         self._load_tracks()
@@ -694,7 +695,7 @@ class Album(Tracklist):
                 "release_type": resp.get("release_type", "album"),
                 "cover_urls": resp.get("image"),
                 "streamable": resp.get("streamable"),
-                "quality": quality_id(
+                "quality": get_quality_id(
                     resp.get("maximum_bit_depth"), resp.get("maximum_sampling_rate")
                 ),
                 "bit_depth": resp.get("maximum_bit_depth"),
@@ -715,8 +716,8 @@ class Album(Tracklist):
                 },
                 "streamable": resp.get("allowStreaming"),
                 "quality": TIDAL_Q_MAP[resp.get("audioQuality")],
-                "bit_depth": 16,
-                "sampling_rate": 44100,
+                "bit_depth": 24 if resp.get("audioQuality") == "HI_RES" else 16,
+                "sampling_rate": 44100,  # always 44.1 kHz
                 "tracktotal": resp.get("numberOfTracks"),
             }
         elif client.source == "deezer":
@@ -726,7 +727,7 @@ class Album(Tracklist):
                 "title": resp.get("title"),
                 "_artist": safe_get(resp, "artist", "name"),
                 "albumartist": safe_get(resp, "artist", "name"),
-                "year": str(resp.get("year"))[:4] or "Unknown",
+                "year": str(resp.get("year"))[:4],
                 # version not given by API
                 "cover_urls": {
                     sk: resp.get(rk)  # size key, resp key
@@ -736,7 +737,7 @@ class Album(Tracklist):
                 },
                 "url": resp.get("link"),
                 "streamable": True,  # api only returns streamables
-                "quality": 6,  # all tracks are 16/44.1 streamable
+                "quality": 2,  # all tracks are 16/44.1 streamable
                 "bit_depth": 16,
                 "sampling_rate": 44100,
                 "tracktotal": resp.get("track_total") or resp.get("nb_tracks"),
@@ -784,11 +785,9 @@ class Album(Tracklist):
     def download(
         self,
         quality: int = 7,
-        parent_folder: Union[str, os.PathLike] = "Downloads",
-        progress_bar: bool = True,
-        tag_tracks: bool = True,
-        cover_key: str = "large",
+        parent_folder: Union[str, os.PathLike] = "StreamripDownloads",
         database: MusicDB = None,
+        **kwargs,
     ):
         """Download all of the tracks in the album.
 
@@ -798,6 +797,11 @@ class Album(Tracklist):
         :type parent_folder: Union[str, os.PathLike]
         :param progress_bar: turn on/off a tqdm progress bar
         :type progress_bar: bool
+        :param large_cover: Download the large cover. This may fail when
+        embedding covers.
+        :param tag_tracks: Tag the tracks after downloading, True by default
+        :param keep_cover: Keep the cover art image after downloading.
+        True by default.
         """
         folder = self._get_formatted_folder(parent_folder)
 
@@ -810,33 +814,45 @@ class Album(Tracklist):
 
         if os.path.isfile(cover_path):
             logger.debug("Cover already downloaded: %s. Skipping", cover_path)
-
         else:
-            if self.cover_urls:
-                cover_url = self.cover_urls.get(cover_key)
+            click.secho("Downloading cover art", fg="magenta")
+            if kwargs.get("large_cover", False):
+                cover_url = self.cover_urls.get("large")
+                if self.client.source == "qobuz":
+                    tqdm_download(cover_url.replace("600", "org"), cover_path)
+                else:
+                    tqdm_download(cover_url, cover_path)
 
-                img = requests.head(cover_url)
-
-                if int(img.headers["Content-Length"]) > FLAC_MAX_BLOCKSIZE:  # 16.7 MB
-                    logger.info(
-                        f"{cover_key} cover size is too large to "
-                        "embed. Using small cover instead"
+                if os.path.getsize(cover_path) > FLAC_MAX_BLOCKSIZE:  # 16.7 MB
+                    click.secho(
+                        "Large cover is too large to embed, embedding small cover instead.",
+                        fg="yellow",
                     )
-                    cover_url = self.cover_urls.get("small")
-
-                tqdm_download(cover_url, cover_path)
+                    large_cover_path = cover_path.replace(".jpg", "_large") + ".jpg"
+                    shutil.move(cover_path, large_cover_path)
+                    tqdm_download(self.cover_urls["small"], cover_path)
+            else:
+                tqdm_download(self.cover_urls["small"], cover_path)
 
         if self.client.source != "deezer":
             cover = self.get_cover_obj(cover_path, quality)
 
+        self.download_message()
         for track in self:
             logger.debug("Downloading track to %s", folder)
-
-            track.download(quality, folder, progress_bar, database=database)
-            if tag_tracks and self.client.source != "deezer":
+            track.download(
+                quality, folder, kwargs.get("progress_bar", True), database=database
+            )
+            if kwargs.get("tag_tracks", True) and self.client.source != "deezer":
                 track.tag(cover=cover)
 
-        logger.debug("Final album folder: %s", folder)
+        if not kwargs.get("keep_cover", True):
+            logger.debug(f"Removing cover at {cover_path}")
+            try:
+                os.remove(cover_path)
+                os.remove(large_cover_path)
+            except Exception as e:
+                logger.debug(e)
 
         self.downloaded = True
 
@@ -889,9 +905,12 @@ class Album(Tracklist):
         """
         return f"{self['albumartist']} - {self['title']}"
 
+    def __len__(self) -> int:
+        return self.tracktotal
+
 
 class Playlist(Tracklist):
-    """Represents a downloadable Qobuz playlist.
+    """Represents a downloadable playlist.
 
     Usage:
     >>> resp = client.get('hip hop', 'playlist')
@@ -938,7 +957,7 @@ class Playlist(Tracklist):
         :param kwargs:
         """
         self.meta = self.client.get(self.id, "playlist")
-        self.name = self.meta.get("name")
+        self.name = self.meta.get("title")
         self._load_tracks(**kwargs)
 
     def _load_tracks(self, new_tracknumbers: bool = True):
@@ -957,7 +976,7 @@ class Playlist(Tracklist):
                 return {"track": track, "album": track["album"]}
 
         elif self.client.source == "tidal":
-            tracklist = self.meta["items"]
+            tracklist = self.meta["tracks"]
 
             def gen_cover(track):
                 cover_url = tidal_cover_url(track["album"]["cover"], 320)
@@ -1018,7 +1037,9 @@ class Playlist(Tracklist):
         """
         folder = sanitize_filename(self.name)
         folder = os.path.join(parent_folder, folder)
+        logger.debug(f"Parent folder {folder}")
 
+        self.download_message()
         for track in self:
             track.download(parent_folder=folder, quality=quality, database=database)
             if self.client.source != "deezer":
@@ -1106,12 +1127,12 @@ class Artist(Tracklist):
         generate album objects and append them to self.
         """
         if self.client.source == "qobuz":
-            self.name = self.meta['name']
+            self.name = self.meta["name"]
             albums = self.meta["albums"]["items"]
 
         elif self.client.source == "tidal":
-            self.name = self.meta['items'][0]['artist']['name']
-            albums = self.meta["items"]
+            self.name = self.meta["name"]
+            albums = self.meta["albums"]
 
         elif self.client.source == "deezer":
             # TODO: load artist name
@@ -1131,6 +1152,7 @@ class Artist(Tracklist):
         no_repeats: bool = False,
         quality: int = 6,
         database: MusicDB = None,
+        **kwargs,
     ):
         """Download all albums in the discography.
 
@@ -1164,9 +1186,8 @@ class Artist(Tracklist):
 
                 final = filter(inter, final)
 
-        i = 0
+        self.download_message()
         for album in final:
-            i += 1
             click.secho(f"Downloading album: {album}", fg="blue")
             try:
                 album.load_meta()
@@ -1176,10 +1197,9 @@ class Artist(Tracklist):
                 parent_folder=folder,
                 quality=quality,
                 database=database,
+                **kwargs,
             )
 
-        logger.debug(f"{i} albums downloaded")
-
     @property
     def title(self):
         return self.name
@@ -1352,7 +1372,6 @@ class Label(Artist):
         resp = self.client.get(self.id, "label")
         self.name = resp["name"]
         for album in resp["albums"]["items"]:
-            pprint(album)
             self.append(Album.from_api(album, client=self.client))
 
     def __repr__(self):
